from enum import Enum


class TransportType(Enum):
    BUS = "bus"
    TRAIN = "mrt"
    TAXI = "taxi"

LTA_SERVICES = {
    "PUBLIC_TRANSPORT_SERVICES": {
        TransportType.BUS: {
            "ARRIVAL": "BusArrivalv2",
            "SERVICES": "BusServices",
            "ROUTES": "BusRoutes",
            "STOPS": "BusStops",
            # "PASSENGER_VOLUME": {
            #     "BUS_STOPS": "PV/Bus",
            #     "ORIGIN_TO_DEST": "PV/ODBus"
            # }
        },
        TransportType.TRAIN: {
            # "PASSENGER_VOLUME": {
            #     "ORIGIN_TO_DEST": "PV/ODTrain",
            # },
            "PLATFORM_CROWD_DENSITY": {
                "REALTIME": "PCDRealTime",
                "FORECAST": "PCDForecast"
            },
            "SERVICE_ALERTS": "TrainServiceAlerts"
        },
        TransportType.TAXI: {
            "AVAILABILITY": "Taxi-Availability",
            "STANDS": "TaxiStands",
        }
}}




from src.ltasg.transport.bus import Bus
from src.ltasg.transport.taxi import Taxi
from src.ltasg.transport.train import Train
import asyncio
b = Bus("http://datamall2.mytransport.sg/ltaodataservice/", "33AWj1PMQdeNJzBFp+YH0Q==")
t = Taxi("http://datamall2.mytransport.sg/ltaodataservice/", "33AWj1PMQdeNJzBFp+YH0Q==")
tr = Train("http://datamall2.mytransport.sg/ltaodataservice/", "33AWj1PMQdeNJzBFp+YH0Q==")
s = asyncio.run(b.stop_services(bus_stop_code=71111))
print(s)



import unittest
from transport.bus import Bus
from transport.train import Train
from transport.taxi import Taxi
import dotenv
import os

class Test(unittest.IsolatedAsyncioTestCase):
    dotenv.load_dotenv()
    api_key = os.getenv("LTA_API_KEY")
    bus = Bus(api_key=api_key)
    train = Train(api_key=api_key)
    taxi = Taxi(api_key=api_key)

    async def test_bus(self):
        print("Start test for bus \n")
        # bus arrival
        arrival_params = [
            [71111, 63],
            [71111, "63M"],
            [71111, 137],
            [71119, 63],
            [71119, "63M"],
            [71119, 137],
        ]
        for param in arrival_params:
            arrival_data = await self.bus.arrival(param[0], param[1])
            print(arrival_data)
            self.assertEqual(str(
                param[1]), arrival_data[0]['ServiceNo'], "Service No is not the same as param")
        # bus services
        services_data = await self.bus.services()
        print(services_data)
        self.assertNotEqual(len(services_data), 0, "Services data is empty")

        # bus routes
        routes_data = await self.bus.routes()
        print(routes_data)
        self.assertNotEqual(len(routes_data), 0, "Routes data is empty")

        # bus stops
        stops_data = await self.bus.stops()
        print(stops_data)
        self.assertNotEqual(len(stops_data), 0, "Stops data is empty")

        print("\Bus test completed\n")

    async def test_train(self):
        print("Start test for train \n")

        # service alerts
        service_alerts_data = await self.train.service_alerts()
        print(service_alerts_data)
        self.assertIn(service_alerts_data['Status'], [
                      1, 2], "Status returned is not valid")
        # platform crowd density realtime

        # platform crowd density forecast

        print("\Train test completed\n")

    async def test_taxi(self):
        print("Start test for taxi \n")

        # Availability
        availability_data = await self.taxi.availability()
        print(availability_data)
        self.assertNotEqual(len(availability_data), 0,
                            "Availability data is empty")
        # stands
        stands_data = await self.taxi.stands()
        print(stands_data)
        self.assertNotEqual(len(stands_data), 0, "Stands data is empty")

        print("\Taxi test completed\n")


if __name__ == "__main__":
    unittest.main()



from ..transport.transport import Transport
from ..constants import TransportType
from ..helpers.helpers import transform_data


class Bus(Transport):
    def __init__(self, base_url, api_key) -> None:
        super().__init__(base_url=base_url,api_key= api_key)
        self.bus_services = self.transport_services[TransportType.BUS]
    
    async def arrival(self, bus_stop_code: int, bus_no: int):
        data = await self.lta_api.fetch(self.bus_services['ARRIVAL'], query_params={
            "BusStopCode": bus_stop_code,
            "ServiceNo": bus_no
        })
        return data["Services"]
    
    async def services(self):
        data = await self.lta_api.fetch(self.bus_services['SERVICES'])
        return transform_data(data)
    async def stop_services(self, bus_stop_code):
        data = transform_data(await self.lta_api.fetch(self.bus_services['ROUTES']))
        services = [ service["ServiceNo"] for service in data if service['BusStopCode'] == str(bus_stop_code)]
        return services
    async def routes(self):
        data = await self.lta_api.fetch(self.bus_services['ROUTES'])
        return transform_data(data)

    async def stops(self):
        data = await self.lta_api.fetch(self.bus_services['STOPS'])
        return transform_data(data)



from ..api import LTADataMall
from ..constants import LTA_SERVICES


class Transport:
    def __init__(self, base_url, api_key) -> None:
        self.lta_api = LTADataMall(base_url=base_url, api_key=api_key)
        self.transport_services = LTA_SERVICES['PUBLIC_TRANSPORT_SERVICES']
    


from aiohttp import ClientSession, ClientConnectionError


class LTADataMall:
    def __init__(self, base_url, api_key) -> None:
        self.BASE_URL = base_url
        self.API_KEY = api_key
        self.HEADERS = {
            "Accept": "application/json",
            "AccountKey": self.API_KEY
        }

    async def fetch(self, service_endpoint: str, query_params: dict = None, data: dict = None):
        url = f"{self.BASE_URL}{service_endpoint}"
        skip_count = 0
        data = []
        if query_params:
            url += "?"
            for param, value in query_params.items():
                url += f"{param}={value}&"
                # if param != list(query_params.keys())[-1]:
                #     url += "&"
        async with ClientSession() as session:
            while True:
                try:
                    url_with_skip = url + f"{"" if query_params else "?"}$skip={skip_count}"
                    async with session.get(url_with_skip, headers=self.HEADERS) as response:
                        if response.ok:
                            curr_data = await response.json()
                            if "Services" in curr_data or isinstance(curr_data['value'], dict):
                                return curr_data
                            if len(curr_data['value']) < 500:
                                data.append(curr_data)
                                break
                            if len(curr_data['value']) == 0:
                                break
                            data.append(curr_data)
                            skip_count += 500
                        else:
                            raise Exception("Error calling api for {} with status code: {}".format(
                                service_endpoint, response.status)) from None
                except ClientConnectionError as e:
                    raise e
            return data




from ..transport.transport import Transport
from ..constants import TransportType

class Train(Transport):
    def __init__(self, base_url, api_key) -> None:
        super().__init__(base_url=base_url,api_key= api_key)
        self.train_services = self.transport_services[TransportType.TRAIN]

    async def service_alerts(self):
        data = await self.lta_api.fetch(self.train_services['SERVICE_ALERTS'])
        return data['value']
    
    # API currently results in Error 404 not found
    # async def platform_crowd_density(self, type: Literal["realtime", "forecast"] = "realtime"):
    #     if type == "realtime":
    #         return await self.lta_api.fetch(self.train_services["PLATFORM_CROWD_DENSITY"]['REALTIME'])
    #     else:
    #         return await self.lta_api.fetch(self.train_services["PLATFORM_CROWD_DENSITY"]["FORECAST"])


from ..transport.transport import Transport
from ..constants import TransportType
from ..helpers.helpers import transform_data

class Taxi(Transport):
    def __init__(self, base_url, api_key) -> None:
        super().__init__(base_url=base_url,api_key= api_key)
        self.taxi_services = self.transport_services[TransportType.TAXI]

    async def availability(self):
        data = await self.lta_api.fetch(self.taxi_services["AVAILABILITY"])
        return transform_data(data)
    
    async def stands(self):
        data = await self.lta_api.fetch(self.taxi_services["STANDS"])
        return transform_data(data)